#include "stm8s.h"
#include "gpio.h"

#define PRS_Hz     128000					 // prescaler output freq.
#define INT_Hz     20             // interrupt freq.
#define ARR 			(int)((float)PRS_Hz/(float)INT_Hz)  // 16bit counter max value

u8 state_left=0, state_right=0;  // 1:up  0:down

main()
{
	CLK_ClockSwitchConfig( CLK_SWITCHMODE_AUTO, CLK_SOURCE_LSI, DISABLE, CLK_CURRENTCLOCKSTATE_DISABLE );

	GPIO_Init(GPIOE, GPIO_PIN_3, GPIO_MODE_IN_PU_NO_IT); // right sw
	GPIO_Init(GPIOE, GPIO_PIN_2, GPIO_MODE_IN_PU_NO_IT); // left sw
	GPIO_Init(GPIOD, GPIO_PIN_7, GPIO_MODE_OUT_PP_HIGH_SLOW); // LED
	GPIO_Init(GPIOD, GPIO_PIN_5, GPIO_MODE_OUT_PP_HIGH_SLOW); // LED
	GPIO_Init(GPIOD, GPIO_PIN_3, GPIO_MODE_OUT_PP_HIGH_SLOW); // LED
	GPIO_Init(GPIOD, GPIO_PIN_0, GPIO_MODE_OUT_PP_HIGH_SLOW); // LED
	GPIO_Init(GPIOE, GPIO_PIN_0, GPIO_MODE_OUT_PP_HIGH_SLOW); // LED

	// bit7531 ---> actuator left H bridge
	// bit6420 ---> actuator right H bridge
	GPIO_Init(GPIOB, 0b11111111, GPIO_MODE_OUT_PP_LOW_FAST);  // actuator

  TIM2_TimeBaseInit( TIM2_PRESCALER_1, ARR-1 );

  TIM2_ITConfig(TIM2_IT_UPDATE, ENABLE);
  TIM2_Cmd(ENABLE);

	state_left =GPIO_ReadPin(GPIOE, GPIO_PIN_2);  // sw left
	state_right=GPIO_ReadPin(GPIOE, GPIO_PIN_3);  // sw right

  enableInterrupts();
	
  while (1) {}
}

u8 sw_left=0, sw_right=0, seqL=0, seqR=0, tmp=0;

@far @interrupt void Tim2Update_isr(void){
  TIM2_ClearFlag(TIM2_FLAG_UPDATE);

	GPIO_WriteReverse(GPIOD,GPIO_PIN_0);

	switch(seqL){    // actuator left control
		case 0: {
			sw_left=GPIO_ReadPin(GPIOE, GPIO_PIN_2);
			if(state_left!=sw_left);
			seqL++; break;
		}
		case 1: {
			sw_left=GPIO_ReadPin(GPIOE, GPIO_PIN_2);
			if(state_left==sw_left){ seqL=0; break; } // no action
			if(sw_left==0){
				GPIO_WriteHigh(GPIOD,GPIO_PIN_7);	// monitor LED
				GPIO_WriteLow (GPIOD,GPIO_PIN_5);	// monitor LED
				tmp = GPIO_ReadInputData(GPIOB);
				tmp = tmp & 0b01010101;
				tmp = tmp | 0b00101000;
				GPIO_Write(GPIOB, tmp); // actuator
			}
			else{
				GPIO_WriteLow (GPIOD,GPIO_PIN_7);	// monitor LED
				GPIO_WriteHigh(GPIOD,GPIO_PIN_5);	// monitor LED
				tmp = GPIO_ReadInputData(GPIOB);
				tmp = tmp & 0b01010101;
				tmp = tmp | 0b10000010;
				GPIO_Write(GPIOB, tmp); // actuator
			}
			state_left=sw_left;
			seqL++; break;
		}
		case 2: {
			GPIO_WriteHigh(GPIOD,GPIO_PIN_7);	// monitor LED
			GPIO_WriteHigh(GPIOD,GPIO_PIN_5);	// monitor LED
			tmp = GPIO_ReadInputData(GPIOB);
			tmp = tmp & 0b01010101;
			GPIO_Write(GPIOB, tmp); // actuator off
			seqL++; break;
		}
		default: {
			GPIO_WriteHigh(GPIOD,GPIO_PIN_7);	// monitor LED
			GPIO_WriteHigh(GPIOD,GPIO_PIN_5);	// monitor LED
			tmp = GPIO_ReadInputData(GPIOB);
			tmp = tmp & 0b01010101;
			GPIO_Write(GPIOB, tmp); // actuator off
			seqL=0; break;
		}
	}

	switch(seqR){   // actuator right control
		case 0: {
			sw_right=GPIO_ReadPin(GPIOE, GPIO_PIN_3);
			if(state_right!=sw_right);
			seqR++; break;
		}
		case 1: {
			sw_right=GPIO_ReadPin(GPIOE, GPIO_PIN_3);
			if(state_right==sw_right){ seqR=0; break; } // no action
			if(sw_right==0){
				GPIO_WriteHigh(GPIOD,GPIO_PIN_3);	// monitor LED
				GPIO_WriteLow (GPIOE,GPIO_PIN_0);	// monitor LED
				tmp = GPIO_ReadInputData(GPIOB);
				tmp = tmp & 0b10101010;
				tmp = tmp | 0b00010100;
				GPIO_Write(GPIOB, tmp); // actuator
			}
			else{
				GPIO_WriteLow (GPIOD,GPIO_PIN_3);	// monitor LED
				GPIO_WriteHigh(GPIOE,GPIO_PIN_0);	// monitor LED
				tmp = GPIO_ReadInputData(GPIOB);
				tmp = tmp & 0b10101010;
				tmp = tmp | 0b01000001;
				GPIO_Write(GPIOB, tmp); // actuator
			}
			state_right=sw_right;
			seqR++; break;
		}
		case 2: {
			GPIO_WriteHigh(GPIOD,GPIO_PIN_3);	// monitor LED
			GPIO_WriteHigh(GPIOE,GPIO_PIN_0);	// monitor LED
			tmp = GPIO_ReadInputData(GPIOB);
			tmp = tmp & 0b10101010;
			GPIO_Write(GPIOB, tmp); // actuator off
			seqR++; break;
		}
		default: {
			GPIO_WriteHigh(GPIOD,GPIO_PIN_3);	// monitor LED
			GPIO_WriteHigh(GPIOE,GPIO_PIN_0);	// monitor LED
			tmp = GPIO_ReadInputData(GPIOB);
			tmp = tmp & 0b10101010;
			GPIO_Write(GPIOB, tmp); // actuator off
			seqR=0; break;
		}
	}


  return;
}